//======================================================================
//-----------------------------------------------------------------------
/**
 * @file		WXProcHook.cpp
 * @brief		֐tbN t@C
 *
 * @author		t.sirayanagi
 * @version		1.0
 *
 * @par			copyright
 * Copyright (C) 2010-2011 Takazumi Shirayanagi\n
 * The new BSD License is applied to this software.
 * see iris_LICENSE.txt
*/
//-----------------------------------------------------------------------
//======================================================================
#define INCG_IRIS_WXProcHook_CPP_

//======================================================================
// include
#include "WXProcHook.h"
#include "WXDebugHelp.h"

namespace iris {
namespace wx {
namespace dbg
{

//======================================================================
// class
/**********************************************************************//**
 *
 * RXgN^
 *
*//***********************************************************************/
CProcHook::CProcHook(void)
: m_pfnOrigin(nullptr)
, m_pfnCurrent(nullptr)
, m_pThunk(nullptr)
{
}

/**********************************************************************//**
 *
 * fXgN^
 *
*//***********************************************************************/
CProcHook::~CProcHook(void)
{
	Close();
}

/**********************************************************************//**
 *
 * ^[Qbg̃I[v
 *
 -----------------------------------------------------------------------
 * @param [in]	pszModuleName	= ^[Qbg֐W[
 * @param [in]	pfnTarget		= ^[Qbg֐|C^
 * @param [in]	hModule			= ^[QbgW[
 * @return	
*//***********************************************************************/
bool CProcHook::Open (LPCTSTR pszModuleName, PROC pfnTarget, HMODULE hModule)
{
#ifdef UNICODE
	return OpenW(pszModuleName, pfnTarget, hModule);
#else
	return OpenA(pszModuleName, pfnTarget, hModule);
#endif
}
/// CProcHook::Open Q
bool CProcHook::OpenA(LPCSTR  pszModuleName, PROC pfnTarget, HMODULE hModule)
{
	Close();

	ULONG ulSize;
	PIMAGE_IMPORT_DESCRIPTOR pidesc = reinterpret_cast<PIMAGE_IMPORT_DESCRIPTOR>(ImageDirectoryEntryToData(
		hModule, TRUE, IMAGE_DIRECTORY_ENTRY_IMPORT, &ulSize));
	if( pidesc == nullptr ) return false;

	while( pidesc->Name )
	{
		LPCSTR lpName = reinterpret_cast<LPCSTR>((PBYTE)hModule + pidesc->Name);
		if( lstrcmpiA(pszModuleName, lpName) == 0 )
			break;
		++pidesc;
	}
	if( pidesc->Name == 0 ) return false;

	PIMAGE_THUNK_DATA pitd = reinterpret_cast<PIMAGE_THUNK_DATA>((PBYTE)hModule + pidesc->FirstThunk);

	while( pitd->u1.Function )
	{
		PROC* ppfn = reinterpret_cast<PROC*>(&pitd->u1.Function);
		if( *ppfn == pfnTarget )
		{
			m_pThunk = pitd;
			m_pfnOrigin = *ppfn;
			return true;
		}
		++pitd;
	}
	return false;

}
/// CProcHook::Open Q
bool CProcHook::OpenW(LPCWSTR pszModuleName, PROC pfnTarget, HMODULE hModule)
{
	Close();

	// TODO : Ή
	return false;
}

/**********************************************************************//**
 *
 * ^[Qbg̃I[v
 *
 -----------------------------------------------------------------------
 * @param [in]	pszModuleName	= ^[Qbg֐W[
 * @param [in]	pszFuncName		= ^[Qbg֐
 * @param [in]	hModule			= ^[QbgW[
 * @return	
*//***********************************************************************/
bool CProcHook::Open (LPCTSTR pszModuleName, LPCTSTR pszFuncName, HMODULE hModule)
{
#ifdef UNICODE
	return OpenW(pszModuleName, pszFuncName, hModule);
#else
	return OpenA(pszModuleName, pszFuncName, hModule);
#endif
}
/// CProcHook::Open Q
bool CProcHook::OpenA(LPCSTR  pszModuleName, LPCSTR  pszFuncName, HMODULE hModule)
{
	HMODULE hDll = GetModuleHandleA(pszModuleName);
	if( hDll == nullptr ) return false;
	PROC pfnTarget = ::GetProcAddress(hDll, pszFuncName);
	return OpenA(pszModuleName, pfnTarget, hModule);
}
/// CProcHook::Open Q
bool CProcHook::OpenW(LPCWSTR pszModuleName, LPCWSTR pszFuncName, HMODULE hModule)
{
	HMODULE hDll = GetModuleHandleW(pszModuleName);
	if( hDll == nullptr ) return false;
	// TODO : Ή
	return false;
	//PROC pfnTarget = ::GetProcAddress(hDll, pszFuncName);
	//return OpenA(pszModuleName, pfnTarget, hModule);
}

/**********************************************************************//**
 *
 * I[vς݂ǂ
 *
 -----------------------------------------------------------------------
 * @return	^Ul
*//***********************************************************************/
bool CProcHook::IsOpen(void)
{
	return m_pThunk != nullptr;
}

/**********************************************************************//**
 *
 * 
 *
*//***********************************************************************/
void CProcHook::Close(void)
{
	Revert();
	m_pThunk = nullptr;
	m_pfnOrigin  = nullptr;
	m_pfnCurrent = nullptr;
}

/**********************************************************************//**
 *
 * u
 *
 -----------------------------------------------------------------------
 * @param [in]	pfnNew			= V֐|C^
 * @return	
*//***********************************************************************/
bool CProcHook::Replace(PROC pfnNew)
{
	if( !IsOpen() ) return false;
	DWORD dwOldProtect;
	PROC* ppfn = reinterpret_cast<PROC*>(&m_pThunk->u1.Function);
	m_pfnPrev = *ppfn;
	VirtualProtect(ppfn, sizeof(ppfn), PAGE_EXECUTE_READWRITE, &dwOldProtect);
	WriteProcessMemory(GetCurrentProcess(), ppfn, &pfnNew, sizeof(pfnNew), nullptr);
	VirtualProtect(ppfn, sizeof(ppfn), dwOldProtect, &dwOldProtect);
	m_pfnCurrent = pfnNew;
	return true;
}

/**********************************************************************//**
 *
 * ɖ߂
 *
 -----------------------------------------------------------------------
 * @return	
*//***********************************************************************/
bool CProcHook::Revert(void)
{
	if( !IsOpen() ) return true;
	if( m_pThunk != nullptr && m_pfnOrigin != nullptr )
	{
		if( !Replace(m_pfnOrigin) ) return false;
	}
	return true;
}


}	// end of namespace dbg
}	// end of namespace wx
}	// end of namespace iris

#if	(defined(_IRIS_SUPPORT_GOOGLETEST) || defined(_IRIS_UNITTEST) || defined(_IRIS_MULTI_UNITTEST))

//======================================================================
// include
#include "unit/gt/gt_inchead.h"
#include "../../os/WXModule.h"
#include "iris_using.h"

static int WINAPI _User_MessageBoxA(HWND hWnd, LPCSTR lpText, LPCSTR lpCaption, UINT uType)
{
	return -1;
}

TEST(ProcHookTest, MessageBox)
{
	{
		::iris::wx::dbg::CProcHook phook;
		ASSERT_TRUE( phook.OpenA("user32.dll", "MessageBoxA", ::GetModuleHandle(nullptr)) );
		ASSERT_TRUE( phook.Replace((PROC)_User_MessageBoxA) );

		int ret = MessageBoxA(nullptr, "ProcHookTest", "test", 0);
		ASSERT_TRUE( ret == -1 );
	}

	{
		::iris::wx::dbg::CProcHook phook;
		ASSERT_TRUE( phook.OpenA("user32.dll", "MessageBoxA", ::GetModuleHandle(nullptr)) );
	}
}

#endif

